/* 
** CSE142, 99 Winter
** Homework 5
** Valentine's Day
**
** Author:     Steve Schmitt
** Student ID: 9827697
** Section:    BD
**	
** There are some major differences between my final program and the example one.
** The biggest difference is that all sub-selections are done via what I call a 
** 'button palette'. This section of small buttons changes depending on what you're
** currently adding / changing. I think it makes for a nicer interface than the
** model program, but it makes things a bit more complicated unfortunately.
** Another difference (besides the number of shapes, etc) is that you can delete
** a shape from the card. Any shapes later than it in the array are "bumped down"
** and the count is decremented.
**
** I wish we had learned about STRUCs before this program, passing 16 parameters
** is a major pain! Most of em are just object_XX, stuff that would be great for
** using strucs.
 */

#include "gp142.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

	/* <time.h> is here only so that one function can be called
	to seed the random number generator.
	Put it here in your program and forget about it, 
	just in case you decide later you need to use random numbers.
	*/

#define TRUE    		1
#define FALSE   		0

/*
 * The state, or status you are in determines what happens when you click 
 * the card:
 * STATUS IDLE			Nothing is happening
 * STATUS CH BG COLOR	The button palette is full of colors; if you click there
 *		 				you'll select a new Background color.
 * STATUS ADD SHAPE		Click in the button palette to select the shape.
 *						Changes to STATUS CLICK ADD when you select one.
 * STATUS ADD MSG		Click in the button palette to select a message.
 *						Changes to STATUS CLICK ADD when you select one.
 * STATUS SELECT		Click on an object on the card to select it for manipulation
 * STATUS SELECTED		An object has been successfully selected
 * STATUS CH COLOR		Click in the button palette to change the object's color
 * STATUS REMOVE		Click on an object on the card to delete it
 * STATUS CLICK ADD		Click on the card to add an instance of the current type
 *						of object selected in the button palette.
 */

#define STATUS_IDLE			0
#define STATUS_CH_BG_COLOR	1
#define STATUS_ADD_SHAPE	2
#define STATUS_ADD_MSG		3
#define STATUS_SELECT		4
#define STATUS_SELECTED		5
#define STATUS_CH_TYPE		6
#define STATUS_CH_COLOR		7
#define STATUS_REMOVE		8
#define STATUS_CLICK_ADD	9

/* Configure the appearance of the buttons on the menu bar 
 * by setting the number of pixels of button height and width
 */
#define BUTTON_PRESSED	 TRUE
#define BUTTON_UNPRESSED FALSE
#define BUTTON_HEIGHT  	35
#define BUTTON_WIDTH 	110
#define SSIZE			5	/* size of the shaded border */
#define XFUDGE			10	/* distance from left edge in pixels */

/* Number of Buttons used in menubar*/
#define NUM_BUTTONS 	7

/* Button palette types, for the choice selections.
** When the palette status is set to one of these, it will draw those
** buttons on the screen and Mousehandler will pay attention to them.
 */
#define PAL_NONE			0
#define PAL_SHAPE			1
#define PAL_OBJECT			2
#define PAL_MESSAGE			3
#define PAL_COLOR			4

#define PAL_WIDTH	100
#define PAL_HEIGHT	20
#define PAL_FUDGE	20	/* gap between palette and regular buttons */
#define PSIZE		SSIZE-1		/* size of shaded border for palette */
					/* note, this must be different from SSIZE
					 * since Draw_Button uses the size to distinguish
					 * button types */
#define CSIZE		PSIZE-1 /* size of the little color buttons */

/* Position of the card in the window */
#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50)

/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0
#define NOFILL 1

/* MAX_COLORS defined in GP142.h already */
#define DEFAULT_COLOR	WHITE

/* Number of objects possible on screen at once.
 *  can modify this value with no problem. */
#define OBJECT_MAX	30

/* Types of objects we support. */
#define OBJECT_NONE		0
#define OBJECT_SHAPE	1
#define OBJECT_MESSAGE	2

/* Number of shapes defined */
#define SHAPE_MAX		5
#define FIRST_SHAPE_NUM 0

/* These are the kinds of shapes we support. */
#define SHAPE_HEART		0
#define SHAPE_STAR		1
#define SHAPE_SMILEY	2
#define SHAPE_DANCER	3
#define SHAPE_MUSIC		4

/* Number of messages defined.*/
#define MSG_MAX			6

/*Possible places for mouse clicks*/
#define CLICK_BUTTON	1  /* You clicked on SOME button (doesn't say which one)*/
#define CLICK_CARD		2  /* You clicked in the card area */
#define CLICK_OUTSIDE	3	/* You clicked outside the card area, not on a button*/	
#define CLICK_PAL		4	/* You clicked on a button palette */


/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);
void Draw_Button(int x, int y, int pressed, int width, int height, 
				 int border, int bgcolor);
void DrawButtons(int press_button);
void DrawPalette(int press_button, int palette);
void DrawCard (int back_color, int status, 
		int object_count, int object_type[], int object_color[], 
		int object_x[], int object_y[], int object_data[]);
void DrawMessage(int msg_choice, int color, int x, int y);
void DrawShape(int color, int shape_type, int bottom_x, int bottom_y);
void DrawHeart (int x, int y, int color);
void DrawStar (int x, int y, int color);
void DrawSmiley(int bx, int by, int color);
void DrawStickDancer(int x, int y, int color);
void DrawMusicNote (int x, int y, int color);

/* Mouse click handling & User interface functions */

int PalToObj(int palette);
int CheckPal(int x, int y, int palette, int pal_count);
int ClassifyClick(int mouse_x, int mouse_y, int palette, int pal_count);
void MouseHandler(int x, int y, int *status, int *palette, int *pal_count, 
				  int *last_button, int *last_pal, int *object_current,
		  		  int *object_count, int object_type[OBJECT_MAX], 
				  int object_color[OBJECT_MAX], int object_x[OBJECT_MAX],
				  int object_y[OBJECT_MAX], int object_data[OBJECT_MAX],
				  int *back_color, int *cur_color);
void ButtonHandler(int y, int *last_button, int *last_pal, int *status,
			int *palette, int *pal_count, int object_type);
void PaletteHandler(int x, int y, int *palette, int *last_pal, int *status, 
			int *back_color, int *object_count, int object_type[],
			int object_color[], int object_x[], int object_y[], 
			int object_data[], int *object_current, int *cur_color);
void CardHandler(int x, int y, int *status, int *palette, int *last_pal,
			int *object_current, int *object_count, int object_type[OBJECT_MAX], 
			int object_color[OBJECT_MAX], int object_x[OBJECT_MAX],
			int object_y[OBJECT_MAX], int object_data[OBJECT_MAX], int *cur_color);


/*****************************************************************************

     MAIN

*****************************************************************************/

/* main() has two chief purposes: to contain the important variables and 
 *initialize them, and to contain the "event loop," which constantly looks 
 *to see if an event has happened, and then processes the event by calling 
 *an appropriate function. An example of an event is a mouse click.
 */
int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
	
	
	/*variables and their default values*/
	int back_color = PURPLE;
	int status = STATUS_IDLE;
	int palette = PAL_NONE;			/* No button palettes active at start. */
	int pal_count = 0;
	int cur_color = DEFAULT_COLOR;	/* The last color selected */
	/* Each object will have a number telling what kind of object it is, a color,
	 * and an x and y coordinate.  You'll need to make these into arrays so you
	 * can support more than one object.
	 */
	int object_current = 0;			/* Which object is selected to manipulate? */
	int object_count = 0;           /* How many objects on the page? */
	int object_type[OBJECT_MAX];    /* What type of object is this?  */
	int object_color[OBJECT_MAX];   /* Color of object               */
	int object_x[OBJECT_MAX];       /* Position of object            */
	int object_y[OBJECT_MAX];
	int object_data[OBJECT_MAX];    /* datafield specifying the shape or message*/ 

	int last_button = -1;           /* Keep track of the last button clicked.
	                                   This would be usefull when highlighting
	                                   the last button the user clicked */
	int last_palette = -1;			/* which button inside the palette is active */

	int event; /*the most recent GP142 event*/
	
	time_t t; /*needed only when initializing a random number generator*/
	

	/*
	 * INITIALIZATIONS
	 */
	srand (time(&t));	    /* Seed the random number generator. This is 
							   something you should do if you plan to use 
							   random numbers. Leave it here in your program 
							   and forget about it.*/
	

	/* These GP142 initializations don't need to be changed */
	GP142_open();           /* Open and init the GP142 Graphics Window      */
	GP142_show_text(1);     /* Hide the text window                         */
	GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */
	
	GP142_clear();	        /* This will clear the window & paint it white. */

  	GP142_animate(ANI_RUN); /* Enable periodic events                       */
	
	printf("Debug Info begins :\n");		/* All printf's go to text window, used for debugging */
	printf("Program start successful\n");
	
	/*
	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this until 
	 * the user signals "quit".
	 */
	while (!quit) {  /*this is the famous "event loop" */

		/*start by redrawing everything */
		DrawBlankScreen ();
		DrawCard (back_color, status, object_count,
			object_type, object_color, object_x, object_y, object_data);
		DrawButtons(last_button);		/* Show the buttons...*/ 
		DrawPalette(last_palette, palette);	/* Show the button palette*/

		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		   appropriately. */
		switch (event) {
		case GP142_QUIT:
			quit = TRUE;           /* set flag to terminate loop           */
			break;
		case GP142_MOUSE:
			/* A mouse click has occurred. Call the mouse handler to 
			   decide where the click was and react as needed. */
			MouseHandler(mouse_x, mouse_y, &status, &palette,
				&pal_count, &last_button, &last_palette, &object_current,
				&object_count, object_type, object_color,
				object_x, object_y, object_data, &back_color, &cur_color);
			break;
		case GP142_PERIODIC:
			/*An events has occurred simply because some time has passed,*/
			/*not currently used in the program*/		

			break;

		default:
			
			break;
			
		}  /*end switch */
		
	}  /* end event loop */
	
	
	/*
	 * WRAP-UP
	 */
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  /*end main */

/*****************************************************************************

     FUNCTIONS

*****************************************************************************/


/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen (void) {
	GP142_rectangleXY (FOREST_GREEN,
					-GP142_XMAX, -GP142_YMAX,
					GP142_XMAX, GP142_YMAX,
					FILL);

}


/*
** This function draws a single button. Input parameters are the coordinates,
** size, bordersize, bg color, and the button's status.
** it draws a nice shaded box, but DOES NOT print a label. This is so
** the same routine can be reused for drawing the color palette.
*/
void Draw_Button(int x, int y, int pressed,
				 int width, int height, int border, int bgcolor) {

	int ch,cb, cm,cl; /* store the colors I want in these */
				  /* color high, color body, color medium, and color low */
	cb = bgcolor;
	if (border == SSIZE) {			/* gray color scheme */
		if (pressed == BUTTON_PRESSED) {
			ch = BLACK;
			cm = WHITE;
			cl = MED_GRAY;
		}
		else {
			ch = WHITE;
			cm = MED_GRAY;
			cl = BLACK;
		}
	}
	else {							/* green color scheme */
		if (pressed == BUTTON_PRESSED) {
			ch = BROWN;
			cm = FOREST_GREEN;
			cl = GREEN;
		}
		else {
			ch = GREEN;
			cm = FOREST_GREEN;
			cl = BROWN;
		}
	}

	GP142_rectangleXY(ch,   /* hilight left edge */
	x,
	y,
	x + border,
	y - height,0);

	GP142_rectangleXY(ch,   /* hilight top edge */
	x,
	y,
	x + width,
	y - border,0);

	GP142_rectangleXY(cm,   /* shade right edge */
	x + width - border,
	y - border,
	x + width,
	y - height,0);

	GP142_rectangleXY(cl,   /* shade bottom edge */
	x + border,
	y - height + border,
	x + width - border,
	y - height,0);

	GP142_triangleXY(cl,	  /* fillet the corners */
	x,
	y - height,
	x + border,
	y - height + border,
	x + border,
	y - height,
	0);

	GP142_triangleXY(cm,		/* another fillet */
	x + width - border,
	y - border,
	x + width,
	y,
	x + width,
	y - border,
	0);

	GP142_triangleXY(cl,		/* yet another */
	x + width - border,
	y - height + border,
	x + width,
	y - height,
	x + width - border,
	y - height,
	0);


	GP142_rectangleXY(cb, /* fill body of the button */
	x + border,
	y - border,
	x + width - border - 1,
	y - height + border + 1, 0);

	GP142_rectangleXY(BLACK,  /* ink outline */
	x,
	y,
	x + width,
	y - height, 1);

}

/* 
 * Draws the buttons on the left side. Currently selected button is
 * shaded to look like it's been depressed. Calls Draw_Button. 
 */
void DrawButtons(int press_button)
{
	char b_labels [NUM_BUTTONS][25] = {
		"Change card color",
		"Add Shape",
		"Add Message",
		"Select object",
		"Change object type",
		"Change object color",
		"Remove object"
	};
	
	int i, x = -GP142_XMAX + XFUDGE, font_size = 12;
	int y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		y = GP142_YMAX - (i * BUTTON_HEIGHT);
		/* draw a nice shaded button*/
		Draw_Button(x, y, (press_button == i), 
			BUTTON_WIDTH, BUTTON_HEIGHT, SSIZE, LT_GRAY);
		/* print the label */
		GP142_printfXY(BLACK, x + SSIZE, y - SSIZE - BUTTON_HEIGHT/2, 
			font_size, b_labels[i]);

	}	
}


/* 
 * Draws the button palette. Will draw a different palette based on the value
 * contained in parameter palette. Hilights selected button also.
 */
void DrawPalette(int press_button, int palette)
{
	int lbl_bgcolor[MAX_COLORS] = { BLACK, WHITE, RED, GREEN, BLUE, YELLOW, MAGENTA,
		CYAN, PURPLE, NAVY_BLUE, DUSTY_PLUM, ICE_BLUE, TURQUOISE, ORANGE,
		BROWN, PINK, CHALK, GOLD, PEACH, FOREST_GREEN, SEA_GREEN, OLIVE,
		MED_GRAY, LT_GRAY};

	char lbl_shape[SHAPE_MAX][25] = {
		"Heart",
		"Star",
		"Smiley",
		"Dancin' Dude",
		"Musical Note",
	};

	char lbl_msg[MSG_MAX][25] = {
		"B Mine!",   /* heh, messages in the vein of those */
		"TRUE LUV",  /* aweful-tasting little candies */
		"I Love you!",
		"Honey",
		"How Romantic",
		"Be my Valentine!"
	};
	
	int i;
	int x = -GP142_XMAX + XFUDGE,y;
	int font_size = 9;
	int xx;

	switch (palette) {
	case PAL_NONE:
		break;
	case PAL_COLOR: /* draw a button with no label, background is the color */
		for (i = 0; i < MAX_COLORS; i++) {
			y = (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT - PAL_FUDGE) - 
				(i / 4)*PAL_HEIGHT;
			xx = x + PAL_WIDTH/4 * (i%4);
			Draw_Button(xx,y, (press_button == i),
				PAL_WIDTH/4, PAL_HEIGHT, CSIZE, lbl_bgcolor[i]);
		}
		break;
	case PAL_SHAPE:	/* draw rectangles and labels for each button */
		for(i = 0; i < SHAPE_MAX; i++) {
			y = (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT - PAL_FUDGE) - i*PAL_HEIGHT;
			/* draw a nice shaded button*/
			Draw_Button(x, y, (press_button == i), 
			PAL_WIDTH, PAL_HEIGHT, PSIZE, SEA_GREEN);
		/* print the label */
			GP142_printfXY(BLACK, x + PSIZE, y - PSIZE - PAL_HEIGHT/2, 
				font_size, lbl_shape[i]);
		}
		break;

	case PAL_MESSAGE:
		for(i = 0; i < MSG_MAX; i++) {
			y = (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT - PAL_FUDGE) - i*PAL_HEIGHT;
			/* draw a nice shaded button*/
			Draw_Button(x, y, (press_button == i), 
			PAL_WIDTH, PAL_HEIGHT, PSIZE, SEA_GREEN);
		/* print the label */
			GP142_printfXY(BLACK, x + PSIZE, y - PSIZE - PAL_HEIGHT/2, 
				font_size, lbl_msg[i]);
		}
		break;

	default:
		break;
	}	
}

/*
 * Render (draw) the page and all its contents.
 */
void DrawCard (int back_color, int status, 
		int object_count, int object_type[], int object_color[], 
		int object_x[], int object_y[], int object_data[])
{
	int font_size = 16;
	int i;

	/* Draw the page background */
	GP142_rectangleXY (back_color,
		PAGE_LLX, PAGE_LLY,
		PAGE_LLX+ PAGE_WIDTH, PAGE_LLY+ PAGE_HEIGHT,
		FILL);
	
	/* 
	 * Draw all card shapes & messages
	 */
	for (i = 0; i < object_count; i++) {
		if (object_type[i] == OBJECT_SHAPE) {
			DrawShape (object_data[i], object_color[i], object_x[i], object_y[i]);
		}
		else if (object_type[i] == OBJECT_MESSAGE) {
			DrawMessage (object_data[i], object_color[i],
				object_x[i], object_y[i]);
		}
	}
	
	/* Write a title above */
	GP142_printfXY (YELLOW, PAGE_LLX+40, PAGE_LLY+PAGE_HEIGHT, font_size+4,
		"Happy Valentine's Day!");

	/*
	 * Write an appropriate status message below the background
	 */

	switch (status) {
	case STATUS_IDLE:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Card Designer awaiting your commands!");
		break;
	case STATUS_CH_BG_COLOR:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Select a backgrond color from the palette");
		break;
	case STATUS_ADD_SHAPE:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Select a shape from the shape palette.");
		break;
	case STATUS_ADD_MSG:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Select a message from the message palette.");
		break;
	case STATUS_SELECT:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click an object on the card to modify.");
		break;
	case STATUS_SELECTED:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Object selected. You can now move the object / modify its attributes.");
		break;
	case STATUS_CH_TYPE:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Select a new object subtype from the palette.");
		break;
	case STATUS_CH_COLOR:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Select a new color for the current object from the palette.");
		break;
	case STATUS_REMOVE:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on an object to remove from the card.");
		break;
	case STATUS_CLICK_ADD:
		GP142_printfXY (WHITE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to place of object of this type.");
		break;
	default:
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer!");
		break;
	} /*end switch */
	
	
} /*end DrawPage */


/*
 * Draw a message. msg_choice is from 0 to MSG_MAX-1, corresponds to the
 * number of the palette button selected.
 */
void DrawMessage(int msg_choice, int color, int x, int y) {
	char msgs[MSG_MAX][25] = {
		"B Mine!",   /* heh, messages in the vein of those */
		"TRUE LUV",  /* aweful-tasting little candies */
		"I Love you!",
		"Honey",
		"How Romantic",
		"Be my Valentine!"
	};

	GP142_printfXY( color, x, y, 14 + (rand() % 2), msgs[msg_choice]);
}


/*
 * Draw one shape.
 */
void DrawShape(int shape_type, int color, int bottom_x, int bottom_y) {

	switch ((shape_type - FIRST_SHAPE_NUM) % SHAPE_MAX) {
	case SHAPE_HEART:
		DrawHeart (bottom_x, bottom_y, color);
		break;
	case SHAPE_STAR:
		DrawStar (bottom_x, bottom_y, color);
		break;
	case SHAPE_SMILEY:
		DrawSmiley (bottom_x, bottom_y, color);
		break;
	case SHAPE_DANCER:
		DrawStickDancer ( bottom_x, bottom_y, color);
		break;
	case SHAPE_MUSIC:
		DrawMusicNote (bottom_x, bottom_y, color);
	default:
		/* Unknown shape selected */
		break;

	} /*end switch*/

}


/*
 * Draw a heart on the card. A heart will be made by two overlapping
 * circles and a triangle.
 *
 * The pulsation is caused by adding a small, random amount to the 
 * heart height each time this function is called.
 */
#define HEART_WIDTH 35
void DrawHeart (int x, int y, int color) {

	int h = (HEART_WIDTH/2) + (rand () %4);

	GP142_circleXY (color, x - h + 5, y, h);
	GP142_circleXY (color, x + h - 5, y, h);
	GP142_triangleXY (color, x- h*2 +5, y-5,
						x + h*2 -5, y-5,
						x, y - h*2 -5, FILL);

}


/*
 * Draw a star on the card.
 *
 * We make the star twinkle by selecting the number of points randomly.  Each 
 * time it is redrawn, it is slightly different than the time before, which 
 * makes it appear to pulsate,
 */
#define STARPOINTS 5
#define STARRADIUS 40
#define PI 3.14159
void DrawStar (int x, int y, int color) {

	int sp;
	double angle;
	int x1, y1;

	sp = STARPOINTS + (rand () %3);
	for (angle = 0.0; angle < 2 * PI; angle = angle + (2*PI)/sp) {
		x1 = (int) (cos (angle) * STARRADIUS);
		y1 = (int) (sin (angle) * STARRADIUS);
		GP142_lineXY (color, x+x1, y+y1, x-x1, y-y1, 1);

	}
}

/* Draw a smiley face on the card. */
#define SMILEY_SIZE 35
void DrawSmiley(int bx, int by, int color) {
	int h = (SMILEY_SIZE/2);
	int x = bx - SMILEY_SIZE/2;
	int y = by - SMILEY_SIZE/2;
	GP142_circleXY (color, x + h , y + h, h);
	GP142_circleXY ((color + 1) % MAX_COLORS, x + h - 5, y + h + 5, 2); /* eyes */
	GP142_circleXY ((color + 1) % MAX_COLORS, x + h + 5, y + h + 5, 2);

	GP142_triangleXY ((color + 2) % MAX_COLORS, 
		x + h -5, y + h - 5, x + h + 5, y + h - 5, x + h + 3, y + h - 8 - rand()%5, 0);
}

/* Draw a dancin' stick dude */
#define STICK_WIDTH 53
#define STICK_HEIGHT 70
void DrawStickDancer(int x, int y, int color) {
	int r1 = (rand() % 16)- 3;
	int r2 = (rand() % 16)- 3;
	x = x - STICK_WIDTH/2;
	y = y - STICK_HEIGHT/2;
	GP142_lineXY (color, x+ r1, y, x+20, y+20, 3); /* legs */
	GP142_lineXY (color, x + 40 +r2, y, x + 20, y + 20, 3);
	GP142_lineXY (color, x+20, y+20, x+20, y+50,3); /* trunk */
	GP142_lineXY (color, x+20, y+40, x + r1, y + 40 - r1, 3); /*arms */
	GP142_lineXY (color, x+20, y+40, x + 40 + r2, y + 40 -r2 ,3);
	GP142_circleXY (color, x+20, y+60, 10); /* head */
}

/* Draw a musical note */
void DrawMusicNote(int x, int y, int color) {
	x = x + (rand() %2) -1;
	y = y + (rand() %2) -1;

	GP142_circleXY (color, x,y,5);
	GP142_lineXY (color, x+5, y, x+5, y + 20, 3);
	GP142_lineXY (color, x+5, y+17, x+12, y+17, 2);
	GP142_lineXY (color, x+5, y+22, x+12, y+22, 2);
}

/*----------------------------------------------------------*
** All Mouse and User-Interface related Routines reside
** after this point.
**----------------------------------------------------------*
*/

/* Calls the appropriate sub-handler function depending on where the mouse
 * was clicked. */
void MouseHandler(int x, int y, int *status, int *palette, int *pal_count, 
				  int *last_button, int *last_pal, int *object_current,
		  		  int *object_count, int object_type[OBJECT_MAX], 
				  int object_color[OBJECT_MAX], int object_x[OBJECT_MAX],
				  int object_y[OBJECT_MAX], int object_data[OBJECT_MAX],
				  int *back_color, int *cur_color)
{
	int cl_where;
	cl_where = ClassifyClick(x, y, *palette, *pal_count);

	if (cl_where == CLICK_BUTTON) {
		ButtonHandler(y, last_button, last_pal, status, palette, pal_count,object_type[*object_current]);
	}
	else if (cl_where == CLICK_PAL) {
		PaletteHandler(x, y, palette, last_pal, status, back_color,
			object_count, object_type, object_color, object_x, object_y,
			object_data, object_current, cur_color);
	}
	else if (cl_where == CLICK_CARD) {
		CardHandler(x,y, status, palette, last_pal, object_current, object_count,
			object_type, object_color, object_x, object_y, object_data, cur_color);
	}
}

/* Given a proper y value, determines which button was pushed + acts
 * appropriately.
 * Original function passed the X value as well, but why do we need it?
 * all relevant checks were performed by ClassifyClick */
void ButtonHandler(int y, int *last_button, int *last_pal, int *status,
				   int *palette, int *pal_count, int object_type) {
	int bnum;

/* Figure out which button was pressed */
	bnum = (GP142_YMAX - y) / BUTTON_HEIGHT;
	switch (bnum) {
	case 0: /* Change background color */
		*palette = PAL_COLOR;
		*pal_count = MAX_COLORS;
		*last_pal = -1;
		*status = STATUS_CH_BG_COLOR;
		break;
	case 1: /* Add Shape */
		*palette = PAL_SHAPE;
		*pal_count = SHAPE_MAX;
		*last_pal = -1;
		*status	= STATUS_ADD_SHAPE;
		break;
	case 2: /* Add Message */
		*palette = PAL_MESSAGE;
		*pal_count = MSG_MAX;
		*last_pal = -1;
		*status = STATUS_ADD_MSG;
		break;
	case 3: /* Select Object */
		*palette = PAL_NONE;
		*pal_count = 0;
		*status = STATUS_SELECT;
		break;
	case 4: /* Change Object subType */
		if (object_type == OBJECT_SHAPE) {
			*palette = PAL_SHAPE;
			*pal_count= SHAPE_MAX;
		}
		else if (object_type == OBJECT_MESSAGE) {
			*palette = PAL_MESSAGE;
			*pal_count = MSG_MAX;
		}
		*last_pal = -1;
		*status = STATUS_CH_TYPE;
		break;
	case 5: /* Change Object Color */
		*palette = PAL_COLOR;
		*pal_count = MAX_COLORS;
		*last_pal = -1;
		*status = STATUS_CH_COLOR;
		break;
	case 6: /* Remove Object */
		*palette = PAL_NONE;
		*pal_count = 0;
		*status = STATUS_REMOVE;
		break;
	default:
		*status = -999;
		break; /* error... */
	}
	printf("Button #%d pushed.\n",bnum);	
	*last_button = bnum;
}

/* Handles any mouse clicks to the button palette. Makes changes to
 * object/background data and changes state as appropriate.
 */
void PaletteHandler(int x, int y, int *palette, int *last_pal, int *status, 
					int *back_color, int *object_count, int object_type[],
					int object_color[], int object_x[], int object_y[], 
					int object_data[], int *object_current, int *cur_color) {
	int bnum;

/* Figure out which palette button was pressed */
	if (*palette == PAL_COLOR) { /* color palette has 4 entries / row */
		bnum = (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT - PAL_FUDGE - y) / PAL_HEIGHT;
		bnum *= 4;
		bnum += (abs(-GP142_XMAX + XFUDGE - x) / (PAL_WIDTH/4));
	}
	else /* the rest are just 1 entry / row */
		bnum = (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT - PAL_FUDGE - y) / PAL_HEIGHT;
	
	printf("Palette Button #%d pushed.\n",bnum); /* debug info */
	if (*status == STATUS_ADD_SHAPE || *status == STATUS_ADD_MSG)
		*status = STATUS_CLICK_ADD;
	if (*status == STATUS_CH_BG_COLOR)
		*back_color = bnum;
	if (*status == STATUS_CH_COLOR) {
		object_color[*object_current] = bnum;
		*cur_color = bnum;
	}
	if (*status == STATUS_CH_TYPE) {
		object_data[*object_current] = bnum;
	}

	*last_pal = bnum;
}

/* This function handlers all mouse clicks to the card. Adds/ Selects/ Removes
 * objects as appropriate. The behavior is controlled by the program status.
 */
void CardHandler(int x, int y, int *status, int *palette, int *last_pal,
		int *object_current, int *object_count, int object_type[OBJECT_MAX], 
		int object_color[OBJECT_MAX], int object_x[OBJECT_MAX],
		int object_y[OBJECT_MAX], int object_data[OBJECT_MAX], int *cur_color)
{
	int i, j, temp;

	if (*status == STATUS_CLICK_ADD) {		/* add an object */
		if ((*object_count+1) < OBJECT_MAX) {
			object_x[*object_count] = x;
			object_y[*object_count] = y;
			object_color[*object_count] = *cur_color;
			object_data[*object_count] = *last_pal;
			temp = PalToObj(*palette);
			if (temp != -1) {
				object_type[*object_count] = temp;
			}
			else {
				*status = 999;
				printf("Error! Error!\n");
			}
			*object_current = *object_count;
			printf("Added object %d of type %d subtype %d", 
				*object_count, object_type[*object_current],
				object_data[*object_current]);
			(*object_count)++;
		}
	}
	else if (*status == STATUS_SELECTED) { /* move an object */
		object_x[*object_current] = x;
		object_y[*object_current] = y;
	}

	else if (*status == STATUS_SELECT) {	/* select an object */
		for (i = 0; i < *object_count; i++) {
			if (x > (object_x[i] - 10) && x < (object_x[i] + 10) &&
				y > (object_y[i] - 10) && y < (object_y[i] + 10))
			{
				*status = STATUS_SELECTED;
				*object_current = i;
				printf("Selected object %d", *object_current);
				break;
			}
		}
	}
	else if (*status == STATUS_REMOVE) { /* remove an object */
		for (i = 0; i < *object_count; i++) {
			if (x > (object_x[i] - 10) && x < (object_x[i] + 10) &&
				y > (object_y[i] - 10) && y < (object_y[i] + 10))
			{
				*object_current = 0;
				printf("Deleted Object %d.", i);
				for (j = i; j < *object_count; j++) {
					object_x[j] = object_x[j+1];
					object_y[j] = object_y[j+1];
					object_type[j] = object_type[j+1];
					object_data[j] = object_data[j+1];
					object_color[j] = object_color[j+1];
				}
				(*object_count)--;
				printf("Object bumpdown occured, now %d objects", *object_count);
				break;
			}
		}
	}

}

/* Quick little function that, given a palette number, returns the type of
 * object that it's for */
int PalToObj(int palette) {
	switch( palette) {
	case PAL_SHAPE:
		return OBJECT_SHAPE;
		break;
	case PAL_MESSAGE:
		return OBJECT_MESSAGE;
		break;
	default:
		return -1; /* error */
		break;
	}
}

/* Checks if mouse coordinates are in the area of the button palette. Acts
 * properly with different sized palettes. Moved to its own function from 
 * ClassifyClick to reduce the clutter.
 */
int CheckPal(int x, int y, int palette, int pal_count)
{
	int a=(GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT - PAL_FUDGE);
	if (palette == PAL_COLOR) {
		if ((x < (-GP142_XMAX + XFUDGE + BUTTON_WIDTH)) &&
			(y < a) &&
			(y >= (a - pal_count/4 * PAL_HEIGHT)))
			return TRUE;
		else
			return FALSE;		
	}
	else {
		if ((x <= (-GP142_XMAX + XFUDGE + BUTTON_WIDTH)) &&
			(y < a) &&
			(y >= (a - pal_count * PAL_HEIGHT)))
			return TRUE;
		else
			return FALSE;
	}
}

/* Figures out what area of the screen the mouse has clicked on, and returns
 * the appropriate value */
int ClassifyClick(int mouse_x, int mouse_y, int palette, int pal_count) {
	if (CheckPal(mouse_x, mouse_y, palette, pal_count))
		return CLICK_PAL;
	else if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
		&& mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
		return CLICK_CARD;
	else if ((mouse_x <= -GP142_XMAX + XFUDGE + BUTTON_WIDTH) &&
      (mouse_y < GP142_YMAX) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT)))
		return CLICK_BUTTON;
	else
		return CLICK_OUTSIDE;
}
